@@ -102,17 +102,19 @@ brasero_tool_color_picker_expose (GtkWidget *widget,
 				  BraseroToolColorPicker *self)
 {
 	BraseroToolColorPickerPrivate *priv;
+	GtkAllocation allocation;
 	cairo_t *ctx;
 
 	priv = BRASERO_TOOL_COLOR_PICKER_PRIVATE (self);
 
-	ctx = gdk_cairo_create (GDK_DRAWABLE (widget->window));
+	ctx = gdk_cairo_create (GDK_DRAWABLE (gtk_widget_get_window (widget)));
 	gdk_cairo_set_source_color (ctx, &priv->color);
+	gtk_widget_get_allocation (widget, &allocation);
 	cairo_rectangle (ctx,
-			 widget->allocation.x,
-			 widget->allocation.y,
-			 widget->allocation.width,
-			 widget->allocation.height);
+			 allocation.x,
+			 allocation.y,
+			 allocation.width,
+			 allocation.height);
 	cairo_fill (ctx);
 	cairo_stroke (ctx);
 	cairo_destroy (ctx);
@@ -141,7 +143,7 @@ brasero_tool_color_picker_response (GtkWidget *widget,
 	priv = BRASERO_TOOL_COLOR_PICKER_PRIVATE (self);
 
 	if (response == GTK_RESPONSE_OK) {
-		selection = GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (priv->dialog)->colorsel);
+		selection = GTK_COLOR_SELECTION (gtk_color_selection_dialog_get_color_selection (GTK_COLOR_SELECTION_DIALOG (priv->dialog)));
 		gtk_color_selection_get_current_color (selection, &priv->color);
 
 		g_signal_emit (self,
@@ -165,10 +167,7 @@ brasero_tool_color_picker_clicked (BraseroToolColorPicker *self,
 	priv = BRASERO_TOOL_COLOR_PICKER_PRIVATE (self);
 
 	dialog = gtk_color_selection_dialog_new (_("Pick a Color"));
-	selection = NULL;
-	g_object_get (dialog,
-	              "color-selection", &selection,
-	              NULL);
+	selection = GTK_COLOR_SELECTION (gtk_color_selection_dialog_get_color_selection (GTK_COLOR_SELECTION_DIALOG (dialog)));
 	gtk_color_selection_set_current_color (selection, &priv->color);
 
 	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (self));
@@ -195,7 +194,6 @@ static void
 brasero_tool_color_picker_init (BraseroToolColorPicker *object)
 {
 	BraseroToolColorPickerPrivate *priv;
-	GtkWidget *frame;
 
 	priv = BRASERO_TOOL_COLOR_PICKER_PRIVATE (object);
 
@@ -208,19 +206,15 @@ brasero_tool_color_picker_init (BraseroToolColorPicker *object)
 	gtk_widget_show (priv->label);
 	gtk_tool_button_set_label_widget (GTK_TOOL_BUTTON (object), priv->label);
 
-	frame = gtk_frame_new (NULL);
-	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_OUT);
-	gtk_widget_show (frame);
-
-	priv->icon = gtk_alignment_new (0.5, 0.5, 1., 1.);
+	priv->icon = gtk_image_new ();
 	gtk_widget_show (priv->icon);
 	g_signal_connect (priv->icon,
 			  "expose-event",
 			  G_CALLBACK (brasero_tool_color_picker_expose),
 			  object);
-	gtk_container_add (GTK_CONTAINER (frame), priv->icon);
 
-	gtk_tool_button_set_icon_widget (GTK_TOOL_BUTTON (object), frame);
+	/* This function expects a GtkMisc object!! */
+	gtk_tool_button_set_icon_widget (GTK_TOOL_BUTTON (object), priv->icon);
 }
 
 static void
