@@ -37,6 +37,7 @@
 #include <gtk/gtk.h>
 
 #include "eggtreemultidnd.h"
+#include "brasero-multi-dnd.h"
 
 #include "brasero-setting.h"
 #include "brasero-file-chooser.h"
@@ -124,50 +125,6 @@ brasero_file_chooser_class_init (BraseroFileChooserClass *klass)
 	object_class->finalize = brasero_file_chooser_finalize;
 }
 
-static void
-brasero_file_chooser_paned_destroy (GObject *object,
-                                                      gpointer NULL_data)
-{
-	if (GPOINTER_TO_INT (g_object_get_data (object, "is-stock-file-chooser")))
-		brasero_setting_set_value (brasero_setting_get_default (),
-		                           BRASERO_SETTING_STOCK_FILE_CHOOSER_PERCENT,
-		                           g_object_get_data (object, "position-percent"));
-	else
-		brasero_setting_set_value (brasero_setting_get_default (),
-		                           BRASERO_SETTING_BRASERO_FILE_CHOOSER_PERCENT,
-		                           g_object_get_data (object, "position-percent"));
-}
-
-static void
-brasero_file_chooser_paned_map_event (GtkWidget *widget,
-                                      GdkEvent *event,
-                                      gpointer NULL_data)
-{
-	gint position;
-	gpointer percent;
-	GtkWidget *toplevel;
-
-	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (widget));
-	if (G_TYPE_FROM_INSTANCE (toplevel) == GTK_TYPE_FILE_CHOOSER_DIALOG) {
-		g_object_set_data (G_OBJECT (widget), "is-stock-file-chooser", GINT_TO_POINTER (1));
-		brasero_setting_get_value (brasero_setting_get_default (),
-		                           BRASERO_SETTING_STOCK_FILE_CHOOSER_PERCENT,
-		                           &percent);
-	}
-	else
-		brasero_setting_get_value (brasero_setting_get_default (),
-		                           BRASERO_SETTING_BRASERO_FILE_CHOOSER_PERCENT,
-		                           &percent);
-
-	if (GPOINTER_TO_INT (percent) < 0) {
-		/* No value so set something sane */
-		percent = GINT_TO_POINTER (30);
-	}
-
-	position = widget->allocation.width * GPOINTER_TO_INT (percent) / 10000;
-	gtk_paned_set_position (GTK_PANED (widget), position);
-}
-
 static void
 brasero_file_chooser_position_percent (GObject *object,
                                        gint width,
@@ -183,7 +140,14 @@ brasero_file_chooser_position_percent (GObject *object,
 	else
 		percent /= width;
 
-	g_object_set_data (object, "position-percent", GINT_TO_POINTER (percent));
+	if (GPOINTER_TO_INT (g_object_get_data (object, "is-stock-file-chooser")))
+		brasero_setting_set_value (brasero_setting_get_default (),
+		                           BRASERO_SETTING_STOCK_FILE_CHOOSER_PERCENT,
+		                           GINT_TO_POINTER (percent));
+	else
+		brasero_setting_set_value (brasero_setting_get_default (),
+		                           BRASERO_SETTING_BRASERO_FILE_CHOOSER_PERCENT,
+		                           GINT_TO_POINTER (percent));
 }
 
 static void
@@ -191,32 +155,81 @@ brasero_file_chooser_position_changed (GObject *object,
                                        GParamSpec *param_spec,
                                        gpointer NULL_data)
 {
+	GtkAllocation allocation = {0, 0};
 	gint position;
-	gint width;
 
+	gtk_widget_get_allocation (GTK_WIDGET (object), &allocation);
 	position = gtk_paned_get_position (GTK_PANED (object));
-	width = GTK_WIDGET (object)->allocation.width;
-	brasero_file_chooser_position_percent (object, width, position);
+	brasero_file_chooser_position_percent (object, allocation.width, position);
 }
 
 static void
-brasero_file_chooser_allocation_changed (GObject *object,
+brasero_file_chooser_allocation_changed (GtkWidget *widget,
                                          GtkAllocation *allocation,
                                          gpointer NULL_data)
 {
 	gint position;
 	gint width;
 
-	position = gtk_paned_get_position (GTK_PANED (object));
+	/* See if it's the first allocation. If so set the position and don't
+	 * save it */
+	if (GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget), "position_set")) == FALSE) {
+		gpointer percent;
+		GtkWidget *toplevel;
+
+		toplevel = gtk_widget_get_toplevel (GTK_WIDGET (widget));
+		if (G_TYPE_FROM_INSTANCE (toplevel) == GTK_TYPE_FILE_CHOOSER_DIALOG) {
+			g_object_set_data (G_OBJECT (widget), "is-stock-file-chooser", GINT_TO_POINTER (1));
+			brasero_setting_get_value (brasero_setting_get_default (),
+				                   BRASERO_SETTING_STOCK_FILE_CHOOSER_PERCENT,
+				                   &percent);
+		}
+		else
+			brasero_setting_get_value (brasero_setting_get_default (),
+				                   BRASERO_SETTING_BRASERO_FILE_CHOOSER_PERCENT,
+				                   &percent);
+
+		if (GPOINTER_TO_INT (percent) >= 0) {
+			position = allocation->width * GPOINTER_TO_INT (percent) / 10000;
+			gtk_paned_set_position (GTK_PANED (widget), position);
+		}
+		else
+			gtk_paned_set_position (GTK_PANED (widget), 30 * allocation->width / 100);
+
+		/* Don't connect to position signal until it was first allocated */
+		g_object_set_data (G_OBJECT (widget), "position_set", GINT_TO_POINTER (TRUE));
+		g_signal_connect (widget,
+		                  "notify::position",
+		                  G_CALLBACK (brasero_file_chooser_position_changed),
+		                  NULL);
+		return;
+	}
+
+	position = gtk_paned_get_position (GTK_PANED (widget));
 	width = allocation->width;
 
-	brasero_file_chooser_position_percent (object, width, position);
+	brasero_file_chooser_position_percent (G_OBJECT (widget), width, position);
+}
+
+static void
+brasero_file_chooser_notify_model (GtkTreeView *treeview,
+                                   GParamSpec *pspec,
+                                   gpointer NULL_data)
+{
+	GtkTreeModel *model;
+
+	model = gtk_tree_view_get_model (treeview);
+	if (model && !EGG_IS_TREE_MULTI_DRAG_SOURCE (model)) {
+		GType type;
+
+		type = G_OBJECT_TYPE (model);
+		brasero_enable_multi_DND_for_model_type (type);
+	}
 }
 
 void
 brasero_file_chooser_customize (GtkWidget *widget, gpointer null_data)
 {
-
 	/* we explore everything until we reach a treeview (there are two) */
 	if (GTK_IS_TREE_VIEW (widget)) {
 		GtkTargetList *list;
@@ -231,8 +244,25 @@ brasero_file_chooser_customize (GtkWidget *widget, gpointer null_data)
 		 * the warning messages we get if we do */
 
 		if (found
-		&&  gtk_tree_selection_get_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW (widget))) == GTK_SELECTION_MULTIPLE)
+		&&  gtk_tree_selection_get_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW (widget))) == GTK_SELECTION_MULTIPLE) {
+			GtkTreeModel *model;
+
+			/* This is done because GtkFileChooser does not use a
+			 * GtkListStore or GtkTreeStore any more. */
 			egg_tree_multi_drag_add_drag_support (GTK_TREE_VIEW (widget));
+			model = gtk_tree_view_get_model (GTK_TREE_VIEW (widget));
+			if (model) {
+				GType type;
+
+				type = G_OBJECT_TYPE (model);
+				brasero_enable_multi_DND_for_model_type (type);
+			}
+			else
+				g_signal_connect (widget,
+				                  "notify::model",
+				                  G_CALLBACK (brasero_file_chooser_notify_model),
+				                  NULL);
+		}
 	}
 	else if (GTK_IS_BUTTON (widget)) {
 		GtkWidget *image;
@@ -247,13 +277,13 @@ brasero_file_chooser_customize (GtkWidget *widget, gpointer null_data)
 		&& (!strcmp (stock_id,GTK_STOCK_ADD)
 		||  !strcmp (stock_id, GTK_STOCK_REMOVE))) {
 			GtkRequisition request;
-			gint width, height;
+			gint width;
 			GtkWidget *parent;
 
 			/* This is to avoid having the left part too small */
-			parent = widget->parent;
-			width = parent->requisition.width;
-			height = parent->requisition.height;
+			parent = gtk_widget_get_parent (widget);
+			gtk_widget_get_requisition (parent, &request);
+			width = request.width;
 			gtk_widget_size_request (parent, &request);
 			if (request.width >= width)
 				gtk_widget_set_size_request (parent,
@@ -279,25 +309,12 @@ brasero_file_chooser_customize (GtkWidget *widget, gpointer null_data)
 					 TRUE);
 			g_object_unref (left);
 
-			g_signal_connect (widget,
-			                  "notify::position",
-			                  G_CALLBACK (brasero_file_chooser_position_changed),
-			                  NULL);
-
 			g_signal_connect (widget,
 			                  "size-allocate",
 			                  G_CALLBACK (brasero_file_chooser_allocation_changed),
 			                  NULL);
-
-			g_signal_connect (widget,
-			                  "destroy",
-			                  G_CALLBACK (brasero_file_chooser_paned_destroy),
-			                  NULL);
-			g_signal_connect (widget,
-			                  "map-event",
-			                  G_CALLBACK (brasero_file_chooser_paned_map_event),
-			                  NULL);
 		}
+
 		gtk_container_foreach (GTK_CONTAINER (widget),
 				       brasero_file_chooser_customize,
 				       NULL);
